﻿-- Work Order Tooltips
-- by Jadya - EU-Well of Eternity

local addonName, addonTable = ...
local L = addonTable.L

local max, sfind, tinsert, tremove = math.max, string.find, table.insert, table.remove

local addon_enabled, garrison
local classIndex = select(3, UnitClass("player"))

local db = addonTable.db

local hookedframes = {}
local pool = {}
local function getframe()
 local f = pool[1]
 if f then
  tremove(pool, 1) 
 else
  f = CreateFrame("Frame")
 end
 return f
end

local frames_backdrop  = { bgFile = "Interface\\Buttons\\GreyscaleRamp64", 
                           edgeFile = "Interface\\Buttons\\WHITE8X8", 
						   edgeSize = 1}

function addonTable:updateTooltipsScale(v)
 for _, tt in pairs(hookedframes) do
  tt:SetScale(v)
 end
end

local lastupdate = 0
local function updateTooltip(tt, elapsed, forced)
 lastupdate = lastupdate + elapsed
 if lastupdate > 1 or forced then lastupdate = 0 else return end

 if tt.id == 0 then
  tt:Clear()
  tt.title:SetText(QUEUE_TIME_UNAVAILABLE)
  return
 end

 local bName, bIcon, _, completedWO, currentWO, _, WOduration, currWOtimeleft, WIname, WIicon
 if tt.t == 1 then
  bName, bIcon, _, completedWO, currentWO, _, WOduration, currWOtimeleft, WIname, WIicon = C_Garrison.GetLandingPageShipmentInfoByContainerID(tt.id)
 elseif tt.t == 2 then
  local talentTrees = C_Garrison.GetTalentTrees(LE_GARRISON_TYPE_7_0, classIndex)
  local completeTalentID = C_Garrison.GetCompleteTalent(LE_GARRISON_TYPE_7_0)
  if talentTrees then
   for treeIndex, tree in ipairs(talentTrees) do
    for talentIndex, talent in ipairs(tree) do
  	 local showTalent = talent.isBeingResearched or talent.id == completeTalentID
  	 if showTalent then
	  bName  = TALENT
 	  WIname = talent.name
	  WIicon = talent.icon
	  currentWO = 1
	  maxWO = 1
	  completedWO = talent.id == completeTalentID and 1 or 0
	  currWOtimeleft = SecondsToTime(talent.researchStartTime + talent.researchDuration - time())
   	 end
    end
   end
  end
 end

 if not bName then
  tt:Clear()
  tt.title:SetText(GARRISON_FOLLOWER_INACTIVE)
  if tt.height == 120 then
   tt:SetHeight(40)
   tt.height = 40
  end
  return
 end
 
 if tt.height == 40 then
  tt:SetHeight(120)
  tt.height = 120
 end
 
 local loot_string = WIname and ("- |T"..WIicon..":12|t "..WIname.." - ") or ""
 local queued_string = currentWO and currentWO - completedWO or 0

 tt.title:SetText("|cffcccc00"..bName.."|r")
 tt.leftfont:SetText(READY..": \n"
                   ..BATTLEFIELD_QUEUE_STATUS..": \n"
                   ..NEXT..": " .."\n\n"
                   ..loot_string)
 tt.rightfont:SetText("|cff00FF00"..(completedWO or 0).."\n"
                    ..queued_string.."\n"
                    ..(currWOtimeleft or NOT_APPLICABLE).."|r")
end

local function buildTooltip(f, t, name)
 if not WOTSaves.npcs[name] then return end

 local id = WOTSaves.npcs[name].id
 local follower_data = ""
 local tt = getframe()

 if not tt.initialized then
  tt:SetSize(150,120)
  tt.height = 120
  tt:SetPoint("BOTTOM", f, "TOP")
  tt:SetBackdrop(frames_backdrop)
  tt:SetBackdropColor(0.07,0.07,0.07,0.9)
  tt:SetBackdropBorderColor(0.5,0.5,0.1,0.9)

  tt.title = tt:CreateFontString()
  tt.title:SetFont("Fonts\\MORPHEUS.ttf", 16, "")
  tt.leftfont = tt:CreateFontString(nil, "OVERLAY", "GameTooltipText")
  tt.rightfont = tt:CreateFontString(nil, "OVERLAY", "GameTooltipText")
  tt.additionaldata = tt:CreateFontString(nil, "OVERLAY", "GameTooltipText")
  tt.title:SetPoint("TOP",0, -5)
  tt.rightfont:SetJustifyH("RIGHT")
  tt.leftfont:SetPoint("TOPLEFT",5,-25)
  tt.rightfont:SetPoint("TOPRIGHT",-5,-25)
  tt.additionaldata:SetPoint("BOTTOM",0, 5)
  
  tt.Clear = function()
   tt.title:SetText("")
   tt.rightfont:SetText("")
   tt.leftfont:SetText("")
   tt.additionaldata:SetText("")
  end
  
  tt.initialized = true
 end

 tt:SetParent(f)
 tt.t = t
 tt.id = id
 tt:Clear()
 tt:SetScale(WOTSaves.scale or 1)

 tt.additionaldata:SetText(follower_data)
 
 tt:SetWidth(math.max(150, #(GARRISON_BUILDING_FOLLOWER_WORKING..(followerName or "")) * 17))
 tt:SetScript("OnUpdate", function(self, elapsed) updateTooltip(self, elapsed) end)
 tt:SetPoint("BOTTOM", f, "TOP")
 updateTooltip(tt, 0, true)
 tt:Show()

 return tt
end

local function nameplateRemoved(id)
 if hookedframes[id] then
  hookedframes[id]:SetParent(nil)
  hookedframes[id]:Hide()
  pool[#pool + 1] = hookedframes[id]
  hookedframes[id] = nil
 end
end

local function nameplateAdded(id)
 local f = C_NamePlate.GetNamePlateForUnit(id)
 
 if not f or not f.UnitFrame.name:IsVisible() then return end

 local name = f.UnitFrame.name:GetText()
 
 if WOTSaves.npcs[name] then
  hookedframes[id] = buildTooltip(f.UnitFrame, WOTSaves.npcs[name].type, name)
 end
end

function checkNamePlates()
 for k,v in pairs(C_NamePlate.GetNamePlates()) do
  if v.UnitFrame.name:IsVisible() then 
   local id = v.UnitFrame.BuffFrame.unit
    local name = v.UnitFrame.name:GetText()
   
   if not hookedframes[id] and WOTSaves.npcs[name] then
    hookedframes[id] = buildTooltip(v.UnitFrame, WOTSaves.npcs[name].type, name)
   end
  end
 end
end

local function setTalentID()
 local name = UnitName("target")
 
 if not WOTSaves.npcs[name] then WOTSaves.npcs[name] = {} end
 local t = WOTSaves.npcs[name]
 t.type = 2
 checkNamePlates()
end

local function setContainerID(containerID)
 local name = UnitName("target")
 local new
 
 if not WOTSaves.npcs[name] then WOTSaves.npcs[name] = {} end
 
 for k,v in pairs(WOTSaves.npcs) do
  if v.id == containerID and k ~= name then
   WOTSaves.npcs[k] = nil
  end
 end

 local t = WOTSaves.npcs[name]
 t.id = containerID
 t.type = 1
 checkNamePlates()
end

function addonTable:nameplate_var_changed(t, v)
 if v == 3 then return end

 local pref = t:sub(1, 3)

 if (pref == "in_" and garrison) or (pref == "out" and not garrison) then
  if t == pref.."_friendly" then
   SetCVar("nameplateShowFriends", v == 1 and 1 or 0)
  elseif t == pref.."_showall" then
   SetCVar("nameplateShowAll",  v == 1 and 1 or 0)
  end
 end
end

local eventframe = CreateFrame("Frame")
local function checkGarrison()

 garrison = nil
 if not C_Garrison.IsUsingPartyGarrison() then
  for i = 2,3 do
   if C_Garrison.IsPlayerInGarrison(i) then
    garrison = i
    break
   end
  end
 end

 if garrison and not addon_enabled then
  eventframe:RegisterEvent("SHIPMENT_CRAFTER_CLOSED")
  eventframe:RegisterEvent("CHAT_MSG_LOOT")
  eventframe:RegisterEvent("CHAT_MSG_CURRENCY")
  eventframe:RegisterEvent("NAME_PLATE_UNIT_ADDED")
  eventframe:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
  eventframe:RegisterEvent("UNIT_TARGET")
  eventframe:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
  eventframe:RegisterEvent("SHIPMENT_CRAFTER_OPENED")
  eventframe:RegisterEvent("GARRISON_TALENT_NPC_OPENED")
  addon_enabled = true
  addonTable:nameplate_var_changed("in__friendly", WOTSaves.in__friendly)
  addonTable:nameplate_var_changed("in__showall", WOTSaves.in__showall)
 elseif addon_enabled and not garrison then
  eventframe:UnregisterEvent("SHIPMENT_CRAFTER_CLOSED")
  eventframe:UnregisterEvent("CHAT_MSG_LOOT")
  eventframe:UnregisterEvent("CHAT_MSG_CURRENCY")
  eventframe:UnregisterEvent("NAME_PLATE_UNIT_ADDED")
  eventframe:UnregisterEvent("NAME_PLATE_UNIT_REMOVED")
  eventframe:UnregisterEvent("UNIT_TARGET")
  eventframe:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
  eventframe:UnregisterEvent("SHIPMENT_CRAFTER_OPENED")
  eventframe:UnregisterEvent("GARRISON_TALENT_NPC_OPENED")
  addonTable:nameplate_var_changed("out_friendly", WOTSaves.out_friendly)
  addonTable:nameplate_var_changed("out_showall", WOTSaves.out_showall)
  addon_enabled = false
 end
end

eventframe:RegisterEvent("VARIABLES_LOADED")
eventframe:RegisterEvent("GARRISON_UPDATE")
local lastT = 0
local function eventhandler(self, event, ...)
 if event == "VARIABLES_LOADED" then
  if not WOTSaves then WOTSaves = {} end
  if not WOTSaves.npcs then WOTSaves.npcs = {} end
  
  WOTSaves.showplates_ingarr = nil
  WOTSaves.changeplates = nil
  
  if WOTSaves.in__showall  == nil then WOTSaves.in__showall  = 3 end
  if WOTSaves.out_showall  == nil then WOTSaves.out_showall  = 3 end
  if WOTSaves.in__friendly == nil then WOTSaves.in__friendly = 1 end
  if WOTSaves.out_friendly == nil then WOTSaves.out_friendly = 2 end

  C_Timer.After(1, function()
    checkGarrison()
	if addon_enabled then
	 --C_Garrison.RequestLandingPageShipmentInfo()
	 checkNamePlates()
	end
   end)
 elseif addon_enabled and (event == "SHIPMENT_CRAFTER_CLOSED" or event == "CHAT_MSG_LOOT" or event == "CHAT_MSG_CURRENCY") and lastT < time() - 1 then
  lastT = time()
  --C_Garrison.RequestLandingPageShipmentInfo()
 elseif event == "GARRISON_UPDATE" then
  checkGarrison()
 elseif event == "NAME_PLATE_UNIT_ADDED" then
  nameplateAdded(...)
 elseif event == "NAME_PLATE_UNIT_REMOVED" then
  nameplateRemoved(...)
 elseif event == "SHIPMENT_CRAFTER_OPENED" then
  setContainerID(...)
 elseif event == "GARRISON_TALENT_NPC_OPENED" then
  setTalentID()
 end
end
eventframe:SetScript("OnEvent", eventhandler)